#! /usr/bin/python

## gui.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    gui
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

gui
===

"""


import os

import wx
import wx.grid
import wx.html

#import trefnydd
from trefnydd.lib.pd import *
from trefnydd.lib.g2p import *
from trefnydd.lib.phones import *

class TreeDisplay(wx.TreeCtrl):
    def __init__(self, parent, debug=False):
        wx.TreeCtrl.__init__(self, parent, -1,
                             style=wx.TR_HAS_BUTTONS|wx.TR_HIDE_ROOT)
        self.parent = parent
        self.debug = debug
        self.root = self.AddRoot('')
        self.selection = None # itemID
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnActivated)
        self.Bind(wx.EVT_CONTEXT_MENU, self.OnShowPopup)
        
    def OnSelChanged(self, evt):
        self.selection = evt.GetItem()

    def OnActivated(self, evt):
        self.selection = evt.GetItem()

    def OnShowPopup(self, event):
        pos = event.GetPosition()
        pos = self.ScreenToClient(pos)
        self.SelectPopup(pos)

    def setParent(self, parent):
        self.parent = parent
        
    def getChildren(self, node, retval='id'):
        result = []
        item, cookie = self.GetFirstChild(node)
        while item:
            if retval == 'id':
                result.append(item)
            elif retval == 'text':
                result.append(self.GetItemText(item))
            elif retval == 'id_text':
                result.append((item, self.GetItemText(item)))
            elif retval == 'obj':
                result.append(self.GetItemPyData(item))
            item, cookie = self.GetNextChild(node, cookie)
        return result

##     def updateDisplay(self):
##         for item in self.getChildren(self.root, 'id'):
##             if self.GetItemPyData(item).changed is True:
##                 pass
            
    def clear(self):
        self.DeleteAllItems()

    def getDirn(self, msg):
        np = wx.DirDialog(self, msg, os.getcwd(), style=wx.DD_DEFAULT_STYLE)
        if np.ShowModal() == wx.ID_OK:
            dirn = np.GetPath()
        np.Destroy()
        return dirn
        
    def getFn(self, msg):
        np = wx.FileDialog(self, msg, os.getcwd(), style=wx.OPEN)
        if np.ShowModal() == wx.ID_OK:
            fn = np.GetPath()
        np.Destroy()
        return fn

    def addItemToNode(self, node, item):
        itemNode = self.AppendItem(node, item.name)
        self.SetItemPyData(itemNode, item)
        # todo: facets - needs work
        if hasattr(item, 'facets'):
            facets = item.getFacets()
            for v in facets.items():
                if self.debug: print v[0]
                facetNode = self.AppendItem(itemNode, v[0])
                self.SetItemPyData(facetNode, v[1])
        else:
            print 'hasattr no facets'


class CorporaDisplay(TreeDisplay):
    def __init__(self, parent, debug=False):
        TreeDisplay.__init__(self, parent, debug)

        self.corpora = [] # ???

        self.popupmenu_corpus = wx.Menu()
        for text in ["Import Data","...?"]:
            item = self.popupmenu_corpus.Append(-1, text)
            self.Bind(wx.EVT_MENU, self.OnPopupItemSelected, item)
        self.popupmenu_corpusItem = wx.Menu()
        item = self.popupmenu_corpusItem.Append(-1, 'View/Edit')
        self.Bind(wx.EVT_MENU, self.OnViewEdit, item)
##         self.Bind(wx.EVT_CONTEXT_MENU, self.OnShowPopup)

    def SelectPopup(self, pos):
        if self.GetItemPyData(self.selection) in self.corpora:
            self.PopupMenu(self.popupmenu_corpus, pos)
        elif not self.GetFirstChild(self.selection)[0]:
            self.PopupMenu(self.popupmenu_corpusItem, pos)

    def OnPopupItemSelected(self, event):
        item = self.popupmenu_corpus.FindItemById(event.GetId())
        text = item.GetText()
        if text == 'Import Data':
            dirn = self.getDirn("Choose Current Location of Data")
            corpus = self.GetItemPyData(self.selection)
            corpus.importData(dirn)
            corpusDataNode = self.getCorpusDataNode(self.selection)
            items = self.getChildren(corpusDataNode, 'obj')
            for item in corpus:
                if item not in items:
                    self.addItemToNode(corpusDataNode, item)
        
    def OnViewEdit(self, event):
        item = self.GetItemPyData(self.selection)
        itemParent = self.GetPyData(self.GetItemParent(self.selection))
        from trefnydd.lib.utterance import SegmentationTier
        if isinstance(item, str):
            # probably audio data
            if self.debug is True:
                print 'audio data?'
        elif isinstance(item, SegmentationTier):
            if self.debug is True:
                print 'SegTier?'
            self.parent.Display(item, itemParent) 
        pass 

    def addCorpus(self, corpus):
        self.corpora.append(corpus)
        corpusNode = self.AppendItem(self.root, corpus.name)
        self.SetItemPyData(corpusNode, corpus)
        if len(corpus): # corpus has data
            dataNode = self.AppendItem(corpusNode, "Data")
            for item in corpus:
                self.addItemToNode(dataNode, item)
            # TODO: add corpus metadata?

    def getCorpusDataNode(self, cNode):
        children = self.getChildren(cNode, 'id_text')
        if 'Data' in [i[1] for i in children]:
            return children[[i[1] for i in children].index('Data')][0]
        else:
            return self.AppendItem(cNode, "Data")


class PD_Display(TreeDisplay):
    """
    Top nodes: grapheme set, phone set, g2p ruleset, wordlist.
    """
    def __init__(self, parent, debug=False):
        TreeDisplay.__init__(self, parent, debug)
        self.pd = PronunciationDictionary(self.debug)

        self.initDisplay()

        self.popupmenu = wx.Menu()
        for text in ["Import from file",
                     "Import from corpora",
                     "...?"]:
            item = self.popupmenu.Append(-1, text)
            self.Bind(wx.EVT_MENU, self.OnPopupItemSelected, item)
            
        self.Bind(wx.EVT_CONTEXT_MENU, self.OnShowPopup)

    def setParent(self, parent):
        self.parent = parent
        self.parent.setPD(self.pd) # todo: danger interlocking objects
       
    def OnShowPopup(self, event):
        pos = event.GetPosition()
        pos = self.ScreenToClient(pos)
        if self.selection in [self.graphemesetNode, self.phonesetNode,
                              self.g2pNode, self.lexNode]:
            self.PopupMenu(self.popupmenu, pos)

##     def SelectPopup(self, pos):
##         if self.selection in self.root.getChildren():
##             self.PopupMenu(self.popupmenu, pos)

##     def addPD(self, pd):
##         self.pd = pd
##         self.addItemToNode(self.root, self.pd)
        
    def OnPopupItemSelected(self, event):
        selection = self.GetItemText(self.selection)
        item = self.popupmenu.FindItemById(event.GetId())
        text = item.GetText()
        if text.endswith('from file'):
            fn = self.getFn('Choose source file for %s' % selection)
            self.pd.importFromFile(selection, fn)
            thing = self.GetItemPyData(self.selection)
            self.SetItemText(self.selection,
                             '%s: %s' % (selection, thing.name))
            items = [(i.name, i) for i in thing]
            items.sort()
            items = [i[1] for i in items]
            for item in items:
                self.addItemToNode(self.selection, item)
        elif text.endswith('from corpora'):
            # only implement word import to pd
            if selection == 'Lexicon':
                wlist = self.parent.getWlistFromCorpora()
                self.pd.importWordList(wlist)
                self.SetItemText(self.selection, 'Lexicon from corpora')
                items = [(i.name, i) for i in self.pd]
                items.sort()
                items = [i[1] for i in items]
                for item in items:
                    self.addItemToNode(self.selection, item)
                
        else: # shouldn't happen
            pass

    def initDisplay(self):
        self.graphemesetNode = self.AppendItem(self.root, 'Grapheme Set')
        self.SetItemPyData(self.graphemesetNode, self.pd.graphemeSet)
        self.phonesetNode = self.AppendItem(self.root, 'Phone Set')
        self.SetItemPyData(self.phonesetNode, self.pd.phoneSet)
        self.g2pNode = self.AppendItem(self.root, 'G2P Rules')
        self.SetItemPyData(self.g2pNode, self.pd.g2pRuleSet)
        self.lexNode = self.AppendItem(self.root, 'Lexicon')
        self.SetItemPyData(self.lexNode, self.pd.entries) # was just pd

